--[[
The MIT License (MIT)

Copyright (c) 2015 Ludvig Alexander Verpe

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
]]--

local namem = {}

--/ Settings
 namem.consonantsFirstChance 		= 4 	-- If set to 4 then there is a 25% chance that first letter will be a consonant, if set to 2 then it's 50%.
 namem.charsBeforeCommon			= 1		-- How many characters from the start of name before producing common pairs.
 namem.charsBeforeDouble			= 3		-- How many characters from the start of name before producing double.
 namem.charsBetweenDouble 			= 2		-- Minimum characters between doubles. 
 namem.doubleConsonantsFrequency 	= 8 	-- The higher the number is the rarer double consonants gets, if set to 0 it won't produce double consonants.
 namem.doubleVowelsFrequency 		= 10 	-- The higher the number is the rarer double vowels gets, if set to 0 it won't produce double vowels.
 namem.commonPairsFrequency			= 6		-- The higher the number is the rarer common pairs gets, if set to 0 it won't produce common pairs.
 namem.lessQXZ 						= true  -- Makes the letters q, x and z appear less in names.
--\

namem.version = 0.4
namem.vowels = {"b","c","d","f","g","h","j","k","l","m","n","p","r","s","t","v","w","x","y","z"}
namem.consonants = {"a","i","e","o","u"}
namem.commonPairsConsonantFirst = {"ee", "aw", "au", "eu", "ow", "oo", "on", "an","er", "in", "es", "im", "am", "as", "os", "et", "ay", "ik", "ir", "ia", "em", "ul", "up", "ad", "ig", "ol", "el", "or", "ot", "ur", "ed", "al", "ar", "og", "us", "un", "en", "ie", "ai", "od", "at", "ut", "it", "ic"}
namem.commonPairsVowelFirst = {"sh", "ps", "nk", "wh", "th", "ld", "nd", "mi", "ng", "rg", "lo", "ds", "da", "go", "ta", "va", "no", "ne", "la", "ce", "re", "rn", "ft", "ge", "ry", "ty", "le", "ly", "se", "me", "rs", "ck", "sk", "gn", "cy", "te", "sa", "st", "ls", "rk", "nu", "nt", "po", "so", "to", "rt", "ze", "de", "ch", "kk", "tt", "dd", "pp", "ll", "ss","nn", "nh", "xt", "ny", "by", "py", "lm", "gs", "ts", "ve", "ns", "ht", "he"}
namem.madeDouble = false
namem.foundType = false
namem.doubleI = 0
namem.seed = os.time()
namem.rng = love.math.newRandomGenerator(namem.seed)
function namem.setRandomSeed(s)
	namem.rng = love.math.newRandomGenerator(s)
	namem.seed = s
end
function namem.getRandomSeed()
	return namem.seed
end
function namem.generateName(l)
	if namem.rng:random(namem.consonantsFirstChance) == 1 then
		namem.lastType = "consonants"
	else
		namem.lastType = "vowels"
	end
	namem.madeDouble = false
	namem.i = 1
	while namem.i <= l do
		if namem.madeDouble and namem.doubleI < namem.i - namem.charsBetweenDouble then
			namem.madeDouble = false
		end
		if namem.lastType == "consonants" then
			if namem.i == 1 then
				namem.lastChar = namem.vowels[namem.rng:random(#namem.vowels)]
				if namem.lessQXZ and (namem.lastChar == "z" or namem.lastChar == "x" or namem.lastChar == "w") then
					namem.lastChar = namem.vowels[namem.rng:random(#namem.vowels)]
				end
				namem.name = string.upper(namem.lastChar)
			else
				namem.lastChar =  namem.vowels[namem.rng:random(#namem.vowels)]
				if namem.lessQXZ and (namem.lastChar == "z" or namem.lastChar == "x" or namem.lastChar == "w") then
					namem.lastChar = namem.vowels[namem.rng:random(#namem.vowels)]
				end
				namem.name = namem.name .. namem.lastChar
			end
			namem.lastType = "vowels"
			namem.i = namem.i + 1
			if namem.commonPairsFrequency > 0 and namem.rng:random(namem.commonPairsFrequency) == 1 and namem.i < l - 1  and namem.i > namem.charsBeforeCommon and not namem.madeDouble then
				namem.name = namem.name .. namem.commonPairsConsonantFirst[namem.rng:random(#namem.commonPairsConsonantFirst)]
				namem.foundType = false
				for i,v in ipairs(namem.consonants) do
					if string.sub(namem.name, -1, -1) == v[i] then
						namem.foundType = true
						break
					end
				end
				if namemFoundType then
					namem.lastType = "consonants"
					namem.madeDouble = true
				else
					namem.lastType = "vowels"
				end
				namem.i = namem.i + 2
			end
			if namem.doubleVowelsFrequency > 0 and namem.rng:random(namem.doubleVowelsFrequency) == 1 and namem.i < l  and namem.i > namem.charsBeforeDouble and not namem.madeDouble then
				namem.lastChar = namem.vowels[namem.rng:random(#namem.vowels)]
				if namem.lessQXZ and (namem.lastChar == "z" or namem.lastChar == "x" or namem.lastChar == "w") then
					namem.lastChar = namem.vowels[namem.rng:random(#namem.vowels)]
				end
				namem.name = namem.name .. namem.lastChar
				namem.i = namem.i + 1
				namem.madeDouble = true
				namem.doubleI = namem.i
				namem.lastType = "vowels"
			end
		elseif namem.lastType == "vowels" then
			if namem.i == 1 then
				namem.lastChar = string.upper(namem.consonants[namem.rng:random(#namem.consonants)])
				namem.name = namem.lastChar
			else
				namem.lastChar = namem.consonants[namem.rng:random(#namem.consonants)]
				namem.name = namem.name .. namem.lastChar
			end
			namem.lastType = "consonants"
			namem.i = namem.i + 1
			if namem.commonPairsFrequency > 0 and namem.rng:random(namem.commonPairsFrequency) == 1 and namem.i < l - 1  and namem.i > namem.charsBeforeCommon and not namem.madeDouble then
				namem.name = namem.name .. namem.commonPairsVowelFirst[namem.rng:random(#namem.commonPairsVowelFirst)]
				namem.foundType = false
				for i,v in ipairs(namem.consonants) do
					if string.sub(namem.name, -1, -1) == v[i] then
						namem.foundType = true
						break
					end
				end
				if namemFoundType then
					namem.lastType = "consonants"
				else
					namem.lastType = "vowels"
					namem.madeDouble = true
				end
				namem.i = namem.i + 2
			end
			if namem.doubleConsonantsFrequency > 0 and namem.rng:random(namem.doubleConsonantsFrequency) == 1 and namem.i < l and namem.i > namem.charsBeforeDouble and not namem.madeDouble then
				namem.lastChar = namem.consonants[namem.rng:random(#namem.consonants)]
				namem.name = namem.name .. namem.lastChar
				namem.i = namem.i + 1
				namem.madeDouble = true
				namem.doubleI = namem.i
				namem.lastType = "consonants"
			end
		end
	end
end
function namem.makeName(l,lm)
	if lm then
		namem.generateName(namem.rng:random(l,lm))
	elseif l then
		namem.generateName(l)
	else
		namem.generateName(namem.rng:random(3,6))
	end
	return namem.name
end
return namem